/* Include files */

#define  INCL_WINSHELLDATA
#define  INCL_VIO
#define  INCL_DOS
#define  INCL_DOSFILEMGR
#define  INCL_DOSMEMMGR
#define  INCL_DOSMISC
#define  INCL_ERRORS
#define  INCL_REXXSAA
#define  _DLL
#define  _MT
#include <os2.h>
#include <rexxsaa.h>
#include <memory.h>
#include <malloc.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>


/*********************************************************************/
/*  Declare all exported functions as REXX functions.                */
/*********************************************************************/
RexxFunctionHandler Rx2Ver;
RexxFunctionHandler Rx2LoadFuncs;
RexxFunctionHandler Rx2DropFuncs;
RexxFunctionHandler Rx2SetFileTimestamp;

/*********************************************************************/
/*  Various definitions used by various functions.                   */
/*********************************************************************/

#define  MAX_DIGITS     9          /* maximum digits in numeric arg  */
#define  MAX            256        /* temporary buffer length        */
#define  IBUF_LEN       4096       /* Input buffer length            */

/*********************************************************************/
/* RxStemData                                                        */
/*   Structure which describes as generic                            */
/*   stem variable.                                                  */
/*********************************************************************/

typedef struct RxStemData {
    SHVBLOCK shvb;                     /* Request block for RxVar    */
    CHAR ibuf[IBUF_LEN];               /* Input buffer               */
    CHAR varname[MAX];                 /* Buffer for the variable    */
                                       /* name                       */
    CHAR stemname[MAX];                /* Buffer for the variable    */
                                       /* name                       */
    ULONG stemlen;                     /* Length of stem.            */
    ULONG vlen;                        /* Length of variable value   */
    ULONG j;                           /* Temp counter               */
    ULONG tlong;                       /* Temp counter               */
    ULONG count;                       /* Number of elements         */
                                       /* processed                  */
} RXSTEMDATA;

/*********************************************************************/
/* RxFncTable                                                        */
/*   Array of names of the REXXUTIL functions.                       */
/*   This list is used for registration and deregistration.          */
/*********************************************************************/

static PSZ  RxFncTable[] =
   {
      "Rx2Ver",
      "Rx2LoadFuncs",
      "Rx2DropFuncs",
      "Rx2SetFileTimestamp"
   };

/*********************************************************************/
/* Numeric Error Return Strings                                      */
/*********************************************************************/

#define  NO_UTIL_ERROR    "0"          /* No error whatsoever        */
#define  ERROR_NOMEM      "2"          /* Insufficient memory        */
#define  ERROR_FILEOPEN   "3"          /* Error opening text file    */

/*********************************************************************/
/* Alpha Numeric Return Strings                                      */
/*********************************************************************/

#define  ERROR_RETSTR   "ERROR:"

/*********************************************************************/
/* Numeric Return calls                                              */
/*********************************************************************/

#define  INVALID_ROUTINE 40            /* Raise Rexx error           */
#define  VALID_ROUTINE    0            /* Successful completion      */

/*********************************************************************/
/* Some useful macros                                                */
/*********************************************************************/

#define BUILDRXSTRING(t, s) { \
  strcpy((t)->strptr,(s));\
  (t)->strlength = strlen((s)); \
}

/*********************************************************************/
/****************  REXXUTIL Supporting Functions  ********************/
/*********************************************************************/

/********************************************************************
* Function:  string2long(string, number)                            *
*                                                                   *
* Purpose:   Validates and converts an ASCII-Z string from string   *
*            form to an unsigned long.  Returns FALSE if the number *
*            is not valid, TRUE if the number was successfully      *
*            converted.                                             *
*                                                                   *
* RC:        TRUE - Good number converted                           *
*            FALSE - Invalid number supplied.                       *
*********************************************************************/

BOOL string2long(PSZ string, LONG *number)
{
  ULONG    accumulator;                /* converted number           */
  INT      length;                     /* length of number           */
  INT      sign;                       /* sign of number             */

  sign = 1;                            /* set default sign           */
  if (*string == '-') {                /* negative?
    sign = -1;                         /* change sign                */
    string++;                          /* step past sign             */
  }

  length = strlen(string);             /* get length of string       */
  if (length == 0 ||                   /* if null string             */
      length > MAX_DIGITS)             /* or too long                */
    return FALSE;                      /* not valid                  */

  accumulator = 0;                     /* start with zero            */

  while (length) {                     /* while more digits          */
    if (!isdigit(*string))             /* not a digit?               */
      return FALSE;                    /* tell caller                */
                                       /* add to accumulator         */
    accumulator = accumulator *10 + (*string - '0');
    length--;                          /* reduce length              */
    string++;                          /* step pointer               */
  }
  *number = accumulator * sign;        /* return the value           */
  return TRUE;                         /* good number                */
}

/*************************************************************************
***             <<<<<< REXXUTIL Functions Follow >>>>>>>               ***
*************************************************************************/

/*************************************************************************
* Function:  Rx2Ver                                                      *
*                                                                        *
* Syntax:    ver = Rx2Ver                                                *
*                                                                        *
* Return:    string - version of RX2UTIL                                 *
*************************************************************************/

ULONG Rx2Ver (CHAR *name, ULONG numargs, RXSTRING args[],
                    CHAR *queuename, RXSTRING *retstr)
{
  BUILDRXSTRING(retstr, "1.0.1");      /* pass back result           */
  return VALID_ROUTINE;                /* no error on call           */
}

/*************************************************************************
* Function:  Rx2LoadFuncs                                                *
*                                                                        *
* Syntax:    call Rx2LoadFuncs [option]                                  *
*                                                                        *
* Params:    none                                                        *
*                                                                        *
* Return:    null string                                                 *
*************************************************************************/

ULONG Rx2LoadFuncs (CHAR *name, ULONG numargs, RXSTRING args[],
                           CHAR *queuename, RXSTRING *retstr)
{
  INT    entries;                      /* Num of entries             */
  INT    j;                            /* Counter                    */

  retstr->strlength = 0;               /* set return value           */
                                       /* check arguments            */

  entries = sizeof(RxFncTable)/sizeof(PSZ);

  for (j = 0; j < entries; j++) 
  {
    RexxRegisterFunctionDll (RxFncTable[j], "RX2UTIL", RxFncTable[j]);
  }
  
  BUILDRXSTRING (retstr, "");
  return VALID_ROUTINE;
}

/*************************************************************************
* Function:  Rx2DropFuncs                                                *
*                                                                        *
* Syntax:    call Rx2DropFuncs                                           *
*                                                                        *
* Return:    NO_UTIL_ERROR - Successful.                                 *
*************************************************************************/

ULONG Rx2DropFuncs(CHAR *name, ULONG numargs, RXSTRING args[],
    CHAR *queuename, RXSTRING *retstr)
{
  INT     entries;                     /* Num of entries             */
  INT     j;                           /* Counter                    */

  if (numargs != 0)                    /* no arguments for this      */
    return INVALID_ROUTINE;            /* raise an error             */

  retstr->strlength = 0;               /* return a null string result*/

  entries = sizeof(RxFncTable)/sizeof(PSZ);

  for (j = 0; j < entries; j++)
    RexxDeregisterFunction (RxFncTable[j]);

  return VALID_ROUTINE;                /* no error on call           */
}

/*************************************************************************
* Function:  Rx2SetFileTimestamp                                         *
*                                                                        *
* Syntax:    call Rx2SetFileTimestamp <day> <month>                      *
*                    <year> <hr> <min> <sec>                             *
*                                                                        *
* Params:    day   - day (1-31)                                          *
*            month - month (1-12)                                        *
*            year  - year (1940-1999)                                    *
*            hr    - hour (0-23)                                         *
*            min   - minutes (0-59)                                      *
*            sec   - seconds (0-59)                                      *
*                                                                        *
* Return:    "0" - success                                               *
*            "1" - failure                                               *
**************************************************************************/

ULONG Rx2SetFileTimestamp 
  (CHAR *name, ULONG numargs, RXSTRING args[], CHAR *queuename, 
   RXSTRING *retstr)
{
  int           j;
  LONG          day, month, year, hr, min, sec;
  ULONG         hf, action, rc;
  FILESTATUS3   fi;

  if (numargs != 7) return INVALID_ROUTINE;
                                       
  if (!string2long(args[1].strptr, &day))   return INVALID_ROUTINE;
  if (!string2long(args[2].strptr, &month)) return INVALID_ROUTINE;
  if (!string2long(args[3].strptr, &year))  return INVALID_ROUTINE;
  if (!string2long(args[4].strptr, &hr))    return INVALID_ROUTINE;
  if (!string2long(args[5].strptr, &min))   return INVALID_ROUTINE;
  if (!string2long(args[6].strptr, &sec))   return INVALID_ROUTINE;

  if (day < 1     || day > 31)     return INVALID_ROUTINE;
  if (month < 1   || month > 12)   return INVALID_ROUTINE;
  if (year < 1940 || year > 1999)  return INVALID_ROUTINE;
  if (hr < 0      || hr > 23)      return INVALID_ROUTINE;
  if (min < 0     || min > 59)     return INVALID_ROUTINE;
  if (sec < 0     || sec > 59)     return INVALID_ROUTINE;
  
  year = year > 1980 ? year-1980 : 0;

  rc = DosOpen (args[0].strptr, &hf, &action, 0L, 0L, 
     OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
     OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYNONE, NULL);
  if (rc) {
     BUILDRXSTRING (retstr, "1");
     return VALID_ROUTINE; }

  rc = DosQueryFileInfo (hf, FIL_STANDARD, &fi, sizeof(fi));
  if (rc) {
     BUILDRXSTRING (retstr, "1");
     goto exiting; }

  fi.fdateCreation.day     = day;
  fi.fdateCreation.month   = month;
  fi.fdateCreation.year    = year;

  fi.ftimeCreation.hours   = hr;
  fi.ftimeCreation.minutes = min;
  fi.ftimeCreation.twosecs = sec/2;

  fi.fdateLastAccess.day     = day;
  fi.fdateLastAccess.month   = month;
  fi.fdateLastAccess.year    = year;

  fi.ftimeLastAccess.hours   = hr;
  fi.ftimeLastAccess.minutes = min;
  fi.ftimeLastAccess.twosecs = sec/2;

  fi.fdateLastWrite.day     = day;
  fi.fdateLastWrite.month   = month;
  fi.fdateLastWrite.year    = year;

  fi.ftimeLastWrite.hours   = hr;
  fi.ftimeLastWrite.minutes = min;
  fi.ftimeLastWrite.twosecs = sec/2;

  rc = DosSetFileInfo (hf, FIL_STANDARD, &fi, sizeof(fi));
  if (rc) {
     BUILDRXSTRING (retstr, "1");
     goto exiting; }

  BUILDRXSTRING (retstr, "0");

exiting:

  rc = DosClose (hf);

  return VALID_ROUTINE;
}
